/*
 * FSpot.Core.ColorManagement.cs
 *
 * Author(s):
 * 	Vasiliy Kirilichev <vasyok@gmail.com>
 *
 * This is free software. See COPYING for details.
 *
 */

using System;
using System.IO;
using System.Collections;

namespace FSpot {
public class ColorManagement {
		public static bool IsEnabled;
		
		private static Cms.Profile display_profile;
		private static Cms.Profile destination_profile;
		private static Cms.Transform standart_transform;
		
		public static FSpot.PhotoImageView PhotoImageView;

		private static string [] search_dir = { "/usr/share/color/icc", "~/.color/icc", "/usr/local/share/color/icc " };
		public static ArrayList Profiles = new ArrayList();
		
		public static Cms.Profile DisplayProfile {
			set {
				display_profile = value;
				//FIXME
//				if ((Cms.IccColorSpace)display_profile.ColorSpace != Cms.IccColorSpace.Cmyk) {
					switch (display_profile.ProductName) {
					case "sRGB built-in - (lcms internal)":
						Preferences.Set (Preferences.COLOR_MANAGEMENT_DISPLAY_PROFILE, "");
						break;
					case "rgb built-in - (lcms internal)":
						Preferences.Set (Preferences.COLOR_MANAGEMENT_DISPLAY_PROFILE, "AlternateRgb");
						break;
					default:
						foreach (Cms.Profile profile in Profiles)
							if (profile.ProductName == display_profile.ProductName)
								Preferences.Set (Preferences.COLOR_MANAGEMENT_DISPLAY_PROFILE, profile.ProductName);
						break;
					}
//				}
//				else {
//					display_profile = Cms.Profile.CreateStandardRgb ();
//				}
			}
			get { return display_profile; }
		}

		public static Cms.Profile DestinationProfile {
			set {
				destination_profile = value;
				//FIXME
				switch (destination_profile.ProductName) {
				case "sRGB built-in - (lcms internal)":
					Preferences.Set (Preferences.COLOR_MANAGEMENT_OUTPUT_PROFILE, "");
					break;
				case "rgb built-in - (lcms internal)":
					Preferences.Set (Preferences.COLOR_MANAGEMENT_OUTPUT_PROFILE, "AlternateRgb");
					break;
				default:
					foreach (Cms.Profile profile in Profiles)
					if (profile.ProductName == destination_profile.ProductName)
						Preferences.Set (Preferences.COLOR_MANAGEMENT_OUTPUT_PROFILE, profile.ProductName);
					break;
					}
			}
			get { return destination_profile; }
		}
		
		public static void GetProfiles (string path)
		{
			//
			if (Directory.Exists (path)) {
				string[] IccColorProfilList = System.IO.Directory.GetFiles (path, "*.icc");
				foreach (string ColorProfilePath in IccColorProfilList) {
					Cms.Profile profile = new Cms.Profile (ColorProfilePath);
					if ((Cms.IccColorSpace)profile.ColorSpace == Cms.IccColorSpace.Rgb)
						Profiles.Add(profile);
				}
				string[] IcmColorProfilList = System.IO.Directory.GetFiles (path, "*.icm");
				foreach (string ColorProfilePath in IcmColorProfilList) {
					Cms.Profile profile = new Cms.Profile (ColorProfilePath);
					if ((Cms.IccColorSpace)profile.ColorSpace == Cms.IccColorSpace.Rgb)
						Profiles.Add(profile);
				}
				string[] DirList = System.IO.Directory.GetDirectories (path);
					foreach (string dir in DirList)
						GetProfiles (dir);
			}
		}
		
		private static void GetSettings ()
		{
			//FIXME: Display
//			Gdk.Screen screen = Gdk.Screen.Default;
//			display_profile = Cms.Profile.GetScreenProfile (screen);
//			if (display_profile.ColorSpace != null) {
				switch (Preferences.Get<string> (Preferences.COLOR_MANAGEMENT_DISPLAY_PROFILE)) {
				case "":
					display_profile = Cms.Profile.CreateStandardRgb ();
					break;
				case "AlternateRgb":
					display_profile = Cms.Profile.CreateAlternateRgb ();
					break;
				default:
				foreach (Cms.Profile profile in Profiles)
					if (profile.ProductName == Preferences.Get<string> (Preferences.COLOR_MANAGEMENT_DISPLAY_PROFILE))
						display_profile = profile;
				if (display_profile == null)
					display_profile = Cms.Profile.CreateStandardRgb ();
					break;
				}
//			}

			//Output
			switch (Preferences.Get<string> (Preferences.COLOR_MANAGEMENT_OUTPUT_PROFILE)) {
			case "":
				destination_profile = Cms.Profile.CreateStandardRgb ();
				break;
			case "AlternateRgb":
				destination_profile = Cms.Profile.CreateAlternateRgb ();
				break;
			default:
				foreach (Cms.Profile profile in Profiles)
					if (profile.ProductName == Preferences.Get<string> (Preferences.COLOR_MANAGEMENT_OUTPUT_PROFILE))
						destination_profile = profile;
				if (destination_profile == null)
					destination_profile = Cms.Profile.CreateStandardRgb ();
				break;
			}
		}
		
		public static void LoadSettings()
		{
			Profiles.Clear();
			foreach (string dir in search_dir)
				GetProfiles (dir);
			GetSettings();
			CreateStandartTransform();
		}
		
		public static void ReloadSettings()
		{
			GetSettings();
			CreateStandartTransform();
			PhotoImageView.Reload();
		}
		
		private static void CreateStandartTransform ()
		{
			Cms.Profile [] list = new Cms.Profile [] { Cms.Profile.CreateStandardRgb (), display_profile };
			
			standart_transform = new Cms.Transform (list,
			                                        Cms.Format.Rgb8,
			                                        Cms.Format.Rgb8,
			                                        Cms.Intent.Perceptual, 0x0000);
		}
		
		//it return the cached (?) default transform
		public static Cms.Transform StandartTransform ()
		{
			if (IsEnabled)
				return standart_transform;
			else
				return null;
		}
		
		//this method create the Cms.Transform using image_profile and current screen profile
		public static Cms.Transform CreateTransform (Gdk.Pixbuf pixbuf, Cms.Profile image_profile)
		{
			if (IsEnabled) {
				if (image_profile == null)
					image_profile = Cms.Profile.CreateStandardRgb ();
		
				Cms.Profile [] list = new Cms.Profile [] { image_profile, display_profile };
				
				Cms.Transform transform = new Cms.Transform (list,
				                                             PixbufUtils.PixbufCmsFormat (pixbuf),
				                                             PixbufUtils.PixbufCmsFormat (pixbuf),
				                                             Cms.Intent.Perceptual, 0x0000);
				return transform;
			}
			else
				return null;
		}
		
		//the main apply color profile to icons. it use the standart image profile and current display profile
		public static void ApplyScreenProfile (Gdk.Pixbuf pixbuf)
		{
			if (IsEnabled && !pixbuf.HasAlpha)
				PixbufUtils.ColorAdjust (pixbuf, pixbuf, standart_transform);
		}
		
		//it works also but it use the image_profile too
		public static void ApplyScreenProfile (Gdk.Pixbuf pixbuf, Cms.Profile image_profile)
		{
			if (IsEnabled && !pixbuf.HasAlpha) {
				if (image_profile == null)
					ApplyScreenProfile (pixbuf);
				else {
					Cms.Profile [] list = new Cms.Profile [] { image_profile, display_profile };
					Cms.Transform transform = new Cms.Transform (list,
					                                             PixbufUtils.PixbufCmsFormat (pixbuf),
					                                             PixbufUtils.PixbufCmsFormat (pixbuf),
					                                             Cms.Intent.Perceptual, 0x0000);
					PixbufUtils.ColorAdjust (pixbuf, pixbuf, transform);
				}
			}
		}
		
//		public static void ApplyScreenProfile (Gdk.Pixbuf src, Gdk.Pixbuf dest)
//		{
//			PixbufUtils.ColorAdjust (src, dest, standart_transform);
//		}
		
		public static void ApplyPrinterProfile (Gdk.Pixbuf pixbuf, Cms.Profile image_profile)
		{
			if (IsEnabled && !pixbuf.HasAlpha) {
				if (image_profile == null)
					image_profile = Cms.Profile.CreateStandardRgb ();
				
				Cms.Profile [] list = new Cms.Profile [] { image_profile, destination_profile };
				
				Cms.Transform transform = new Cms.Transform (list,
				                                             PixbufUtils.PixbufCmsFormat (pixbuf),
				                                             PixbufUtils.PixbufCmsFormat (pixbuf),
				                                             Cms.Intent.Perceptual, 0x0000);
				PixbufUtils.ColorAdjust (pixbuf, pixbuf, transform);
			}
		}
	}
}
